=== modified file 'colordiff.py'
--- colordiff.py	2008-02-13 04:58:32 +0000
+++ colordiff.py	2008-04-19 15:10:17 +0000
@@ -15,13 +15,14 @@
 #    along with this program; if not, write to the Free Software
 #    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
+import re
 import sys
 from os.path import expanduser
 
 from bzrlib import patiencediff, trace
 from bzrlib.commands import get_cmd_object
 from bzrlib.patches import (hunk_from_header, InsertLine, RemoveLine,
-                            ContextLine, Hunk)
+                            ContextLine, Hunk, HunkLine)
 
 import terminal
 
@@ -42,11 +43,14 @@
 class DiffWriter(object):
 
     colors = {
-        'metaline':    'darkyellow',
-        'plain':       'darkwhite',
-        'newtext':     'darkblue',
-        'oldtext':     'darkred',
-        'diffstuff':   'darkgreen'
+        'metaline':      'darkyellow',
+        'plain':         'darkwhite',
+        'newtext':       'darkblue',
+        'oldtext':       'darkred',
+        'diffstuff':     'darkgreen',
+        'trailingspace': 'yellow',
+        'leadingtabs':   'magenta',
+        'longline':      'cyan',
     }
 
     def __init__(self, target, check_style):
@@ -54,6 +58,7 @@
         self.lp = LineParser()
         self.chunks = []
         self._read_colordiffrc()
+        self.added_leading_tabs = 0
         self.added_trailing_whitespace = 0
         self.spurious_whitespace = 0
         self.long_lines = 0
@@ -87,10 +92,23 @@
 
             self.colors[key] = val
 
-    def colorstring(self, type, string):
+    def colorstring(self, type, item, bad_ws_match):
         color = self.colors[type]
         if color is not None:
-            string = terminal.colorstring(str(string), color)
+            if self.check_style and bad_ws_match:
+                #highlight were needed
+                item.contents = ''.join(terminal.colorstring(txt, color, bcol)
+                    for txt, bcol in (
+                        (bad_ws_match.group(1).expandtabs(),
+                             self.colors['leadingtabs']),
+                        (bad_ws_match.group(2)[0:self.max_line_len], None),
+                        (bad_ws_match.group(2)[self.max_line_len:],
+                             self.colors['longline']),
+                        (bad_ws_match.group(3), self.colors['trailingspace'])
+                    )) + bad_ws_match.group(4)
+            string = terminal.colorstring(str(item), color)
+        else:
+            string = str(item)
         self.target.write(string)
 
     def write(self, text):
@@ -102,27 +120,37 @@
 
     def _writeline(self, line):
         item = self.lp.parse_line(line)
+        bad_ws_match = None
         if isinstance(item, Hunk):
             line_class = 'diffstuff'
             self._analyse_old_new()
-        elif isinstance(item, InsertLine):
-            if item.contents.endswith(' \n'):
-                self.added_trailing_whitespace += 1
-            if (len(item.contents.rstrip('\n')) > self.max_line_len and
-                not item.contents.startswith('++ ')):
-                self.long_lines += 1
-            line_class = 'newtext'
-            self._new_lines.append(item)
-        elif isinstance(item, RemoveLine):
-            line_class = 'oldtext'
-            self._old_lines.append(item)
+        elif isinstance(item, HunkLine):
+            bad_ws_match = re.match(r'^([\t]*)(.*?)([\t ]*)(\r?\n)$',
+                                    item.contents)
+            has_leading_tabs = bool(bad_ws_match.group(1))
+            has_trailing_whitespace = bool(bad_ws_match.group(3))
+            if isinstance(item, InsertLine):
+                if has_leading_tabs:
+                    self.added_leading_tabs += 1
+                if has_trailing_whitespace:
+                    self.added_trailing_whitespace += 1
+                if (len(bad_ws_match.group(2)) > self.max_line_len and
+                    not item.contents.startswith('++ ')):
+                    self.long_lines += 1
+                line_class = 'newtext'
+                self._new_lines.append(item)
+            elif isinstance(item, RemoveLine):
+                line_class = 'oldtext'
+                self._old_lines.append(item)
+            else:
+                line_class = 'plain'
         elif isinstance(item, basestring) and item.startswith('==='):
             line_class = 'metaline'
             self._analyse_old_new()
         else:
             line_class = 'plain'
             self._analyse_old_new()
-        self.colorstring(line_class, str(item))
+        self.colorstring(line_class, item, bad_ws_match)
 
     def flush(self):
         self.target.flush()
@@ -160,6 +188,9 @@
     finally:
         sys.stdout = real_stdout
     if check_style:
+        if dw.added_leading_tabs > 0:
+            trace.warning('%d new line(s) have leading tabs.' %
+                          dw.added_leading_tabs)
         if dw.added_trailing_whitespace > 0:
             trace.warning('%d new line(s) have trailing whitespace.' %
                           dw.added_trailing_whitespace)

